package org.netkernel.layer0.bnf;


import java.io.IOException;
import java.io.StringWriter;
import org.netkernel.container.IMessages;
import org.netkernel.layer0.meta.GrammarException;
import org.netkernel.layer0.meta.IIdentifierGrammar;
import org.netkernel.layer0.util.OrderedCheapMultiStringMap;

class GrammarImpl implements IIdentifierGrammar
{
	private IGrammarGroup mRoot;
	private String mStartsWith;
	
	public GrammarImpl(IGrammarGroup aRoot, String aStartsWith)
	{	mRoot=aRoot;
		mStartsWith=aStartsWith;
	}
	
	public String toString()
	{	
		StringBuilder sb = new StringBuilder(128);
		recurseToString(mRoot,sb);
		return sb.toString();
	}
	
	public String toXMLString()
	{
		StringWriter sw = new StringWriter(1024);
		try
		{	mRoot.writeXML(sw);
		}
		catch (IOException e) {}
		return sw.toString();
	}
	
	private void recurseToString(IGrammarGroup aGroup, StringBuilder sb)
	{
		for (int i=0; i<aGroup.getPartCount(); i++)
		{
			IGrammarPart p=aGroup.getPart(i);
			if (p instanceof StaticPart)
			{	sb.append(((StaticPart)p).getContent());
				sb.append(' ');
			}
			else if (p instanceof RegexPart)
			{	sb.append("* ");
			}
			else if (p instanceof IGrammarGroup)
			{	recurseToString((IGrammarGroup)p, sb);
			}
		}
	}
	
	public boolean matches(String aString)
	{	boolean result=false;
		if (mStartsWith==null || aString.startsWith(mStartsWith))
		{	String[] groups=mRoot.matchesForGroups(aString);
			if (groups!=null)
			{	result = !mRoot.needsToMatchDeeper() || mRoot.matches(groups,1);
			}
		}
		return result;
	}
	

	public boolean partMatches(String aName, String aValue)
	{		
		boolean result=false;
		IGrammarGroup g=findGroupWithName(aName,mRoot);
		if (g==null)
		{	g=findGroupWithName(BNFGrammarFactory.ARG_UNDEFINED_VALUE,mRoot);
		}
		if (g!=null)
		{	result = g.match(aValue);
		}
	
		return result;
	}
	
	private static IGrammarGroup findGroupWithName(String aName, IGrammarGroup aGroup)
	{	IGrammarGroup result=null;
		String name=aGroup.getName();
		if (aName.equals(name))
		{	result=aGroup;
		}
		else
		{	for (int i=0; i<aGroup.getPartCount() && result==null; i++)
			{	Object part=aGroup.getPart(i);
				if (part instanceof IGrammarGroup)
				{	result = findGroupWithName(aName,(IGrammarGroup)part);
				}
			}
		}
		return result;
	}
	

	public OrderedCheapMultiStringMap parse(String aString)
	{	OrderedCheapMultiStringMap results;
	
		String[] groups=mRoot.matchesForGroups(aString);
		if (groups!=null)
		{	results=new OrderedCheapMultiStringMap(8);
			mRoot.parse(groups, results, 1);
		}
		else
		{	results=OrderedCheapMultiStringMap.EMPTY;
		}	
		return results;
	}

	public String construct(OrderedCheapMultiStringMap aParts, IMessages aMessages) throws GrammarException
	{	OrderedCheapMultiStringMap parts=new OrderedCheapMultiStringMap(aParts);
		String result=mRoot.constructIdentifier(parts, aMessages);
		if (parts.size()>0)
		{	throw new ExtraneousPartsException(aMessages.format("MSG_BNF_EXTRANEOUS", parts.toString()),parts);
		}
		return result;
	}
	
	static String locateClosestName(IGrammarGroup aGroup)
	{	String name="<root>";
		while (aGroup!=null)
		{	String name2=aGroup.getName();
			if (name2!=null)
			{	name=name2;
				break;
			}
			aGroup=aGroup.getParent();
		}
		return name;
	}
}