package org.netkernel.layer0.urii;

import org.netkernel.container.IKernel;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.container.config.IConfigurationListener;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.INetKernelException;
import org.netkernel.urii.ISpace;

public abstract class EndpointImpl implements IEndpoint
{
	private IKernel mKernel;
	private ISpace mSpace;
	private boolean mThreadSafe;
	private boolean mUnhandledExceptionsExpired;
	private boolean mVerboseUnhandledExceptions;
	
	private static boolean sThreadSafeDefault=false;
	private static IConfigurationListener sConfigurationListener;
	
	public EndpointImpl()
	{	mThreadSafe=sThreadSafeDefault;
		mUnhandledExceptionsExpired=true;
		mVerboseUnhandledExceptions=true;
	}
	
	public EndpointImpl(boolean aThreadSafe)
	{	mThreadSafe=aThreadSafe;
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	mKernel=aKernel;
		mSpace=aSpace;
		
		if (sConfigurationListener==null)
		{	 sConfigurationListener = new IConfigurationListener()
			{	public void configurationChanged(IConfiguration aConfiguration)
				{	try
					{	sThreadSafeDefault = aConfiguration.getBoolean("netkernel.threadsafe.endpoint.default");
					}
					catch (Exception e) {;	}	
				}
			};
			aKernel.addConfigurationListener(sConfigurationListener);
		}
		
	}

	public void onDecommissionEndpoint() throws Exception
	{	mKernel=null;
		mSpace=null;
	}

	/** Marks the endpoint to be considered safe to process more than one request concurrently.
	 * By default this is not the case and the kernel will only allow one request at a time to enter
	 * the endpoint. This restriction does not apply to META and RESOLVE requests.
	 */
	protected void declareThreadSafe()
	{	mThreadSafe=true;
	}
	
	/** Marks the endpoint to be considered safe to process more than one request concurrently.
	 * By default this is not the case and the kernel will only allow one request at a time to enter
	 * the endpoint. This restriction does not apply to META and RESOLVE requests.
	 */
	protected void declareThreadSafe(boolean aSafe)
	{	mThreadSafe=aSafe;
	}	
	
	/** Sets the expiration behaviour of unhandled exceptions thrown during request handling.
	 * By default unhandled exceptions are expired such that the endpoint will be retried every time. However
	 * it is often valid for the exception to be cachable if no external dependencies could change
	 * the outcome. 
	 * @param aExpired. If true unhandled exceptions will always be expired regardless of other determinants,
	 *  if false then the exception response will remain valid until other determinants cause invalidation
	 */
	protected void declareUnhandledExceptionsExpired(boolean aExpired)
	{	mUnhandledExceptionsExpired=aExpired;
	}
	
	protected void declareVerboseUnhandledExceptions(boolean aVerbose)
	{	mVerboseUnhandledExceptions=aVerbose;
	}
	
	
	protected IKernel getKernel()
	{	return mKernel;
	}
	
	protected ISpace getSpace()
	{	return mSpace;
	}

	public boolean isThreadSafe()
	{	return mThreadSafe;
	}
	
	public boolean areUnhandledExceptionsExpired()
	{	return mUnhandledExceptionsExpired;
	}
	
	public boolean areUnhandledExceptionsVerbose()
	{	return mVerboseUnhandledExceptions;
	}
}
